package com.zhongshu.reward.server.core.service;

import com.github.microservice.auth.client.content.ResultState;
import com.github.microservice.productcenter.client.model.ProductRuleModel;
import com.github.microservice.productcenter.client.ret.ResultContent;
import com.github.microservice.productcenter.client.service.ProductRuleService;
import com.zhongshu.payment.client.model.order.v2.GoodsResultModel;
import com.zhongshu.payment.client.service.v2.OrderService2;
import com.zhongshu.reward.client.type.ReceiptsStatus;
import com.zhongshu.reward.client.type.ReceiptsType;
import com.zhongshu.reward.server.core.dao.WalletDao;
import com.zhongshu.reward.server.core.dao.WalletReceiptsDao;
import com.zhongshu.reward.server.core.domain.Wallet;
import com.zhongshu.reward.server.core.domain.WalletReceipts;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.math.BigDecimal;
import java.util.Date;
import java.util.List;
import java.util.stream.Collectors;

/**
 * @author wjf
 * @date 2024/8/23
 */
@Service
@Slf4j
public class GoodsReceiptsService {

    @Autowired
    WalletDao walletDao;

    @Autowired
    WalletService walletService;

    @Autowired
    WalletReceiptsDao walletReceiptsDao;

    @Autowired
    ProductRuleService productRuleService;

    @Autowired
    OrderService2 orderService2;

    /**
     * 购物返利 todo// 1、获取产品库 返利规则 2、获取订单详情、获取商品详情
     */
    @Transactional
    public void receipts(String userId, String orderNo, String payGoodsId, Long operationTime){

        com.github.microservice.auth.client.content.ResultContent<GoodsResultModel> goodsResultModelResultContent = orderService2.qrGoods(payGoodsId);
        if (!ResultState.Success.equals(goodsResultModelResultContent.getState())){
            log.info("查询商品信息失败：id:{}", payGoodsId);
            return;
        }
        GoodsResultModel goodsResultModel = goodsResultModelResultContent.getContent();
        if (StringUtils.isEmpty(goodsResultModel.getGoodsId())){
            log.info("商品无对应产品库id:{}", payGoodsId);
            return;
        }
        ProductRuleModel productRule = getRule(goodsResultModel.getGoodsId());
        if (null == productRule){
            return;
        }

        BigDecimal total = getReceiptsTotal(goodsResultModel.getPayAmount(), productRule);
        if (BigDecimal.ZERO.compareTo(total) == 0){
            log.info("商品返利金额为0, id:{}, productId:{}", payGoodsId, goodsResultModel.getGoodsId());
            return;
        }

        Wallet wallet = walletService.getWalletByUserId(userId);

        WalletReceipts walletReceipts = new WalletReceipts();
        walletReceipts.setWallet(wallet);
        walletReceipts.setReceiptsType(ReceiptsType.REBATE);
        walletReceipts.setEstimatedTime(operationTime + 2*24*60*60*1000);
        walletReceipts.setTotal(total);
        walletReceipts.setOutTradeNo(orderNo);
        walletReceipts.setGoodsId(goodsResultModel.getGoodsId());
        walletReceipts.setPaymentGoodsId(payGoodsId);
        walletReceipts.setStatus(ReceiptsStatus.WAIT);

        walletReceiptsDao.save(walletReceipts);
        walletDao.updateIncWaitAmount(wallet.getId(), total);
        log.info("购物返利入账");
    }

    @NotNull
    private static BigDecimal getReceiptsTotal(BigDecimal goodsTotal, ProductRuleModel productRule) {
        BigDecimal total = BigDecimal.ZERO;

        if (productRule.getCashBackFixed()!=null){
            total = productRule.getCashBackFixed();
        }else if (productRule.getCashBackPercent()!=null){
            total = goodsTotal.multiply(productRule.getCashBackPercent().multiply(BigDecimal.TEN.pow(2)));
        } return total;
    }

    @Nullable
    private ProductRuleModel getRule(String goodsId) {
        ResultContent<ProductRuleModel> goodsRulerResult = productRuleService.list(goodsId);
        ResultContent<ProductRuleModel> overRulerResult = productRuleService.list(null);

        ProductRuleModel productRule = goodsRulerResult.getContent();
        ProductRuleModel overRule = overRulerResult.getContent();
        if (productRule.getEnableCashback()!=null && productRule.getEnableCashback()){
            return productRule;
        }else if (overRule.getEnableCashback()!=null && overRule.getEnableCashback()){
            return overRule;
        }
        return null;
    }


    /**
     * 取消购物返利
     * @param orderNo 订单号
     */
    @Transactional
    public void cancel(String orderNo, String payGoodsId, Long operateTime){
        WalletReceipts walletReceipts = walletReceiptsDao.findByOutTradeNoAndPaymentGoodsIdAndStatus(orderNo, payGoodsId, ReceiptsStatus.WAIT);
        if (operateTime > walletReceipts.getEstimatedTime()){
            return;
        }
        walletReceipts.setStatus(ReceiptsStatus.CANCEL);
        walletReceiptsDao.save(walletReceipts);
        walletDao.updateIncWaitAmount(walletReceipts.getWallet().getId(), walletReceipts.getTotal().negate());
    }

    /**
     * 入账结算
     */
    @Transactional
    public void settle(){
        //todo 查询到期入账的订单
        WalletReceipts walletReceipts = walletReceiptsDao.findTop1ById("");

        walletReceipts.setReceiptsTime(new Date().getTime());
        walletReceipts.setStatus(ReceiptsStatus.RECEIPTS);
        walletReceiptsDao.save(walletReceipts);
        walletDao.updateIncWaitAmount(walletReceipts.getWallet().getId(), walletReceipts.getTotal().negate());
        walletDao.updateIncAmount(walletReceipts.getWallet().getId(), walletReceipts.getTotal());
    }
}
